@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import logging
 import reprlib
 from bisect import bisect_left, bisect_right
 from itertools import chain
@@ -9,12 +8,13 @@ from typing import TYPE_CHECKING, Dict, Iterable, List, Union
 
 import numpy as np
 
+from ada.base.units import Units
 from ada.concepts.exceptions import DuplicateNodes
 from ada.concepts.points import Node, replace_node
 from ada.concepts.stru_beams import Beam
 from ada.concepts.stru_plates import Plate
 from ada.concepts.transforms import Rotation
-from ada.config import Settings
+from ada.config import Settings, get_logger
 from ada.core.utils import Counter, roundoff
 from ada.core.vector_utils import (
     is_null_vector,
@@ -28,8 +28,11 @@ from ada.materials import Material
 if TYPE_CHECKING:
     from ada import FEM, Assembly, Part
     from ada.concepts.connections import JointBase
+    from ada.fem.results.common import FemNodes
     from ada.sections import Section
 
+logger = get_logger()
+
 __all__ = [
     "Nodes",
     "Beams",
@@ -55,7 +58,6 @@ class Beams(BaseCollections):
     """A collections of Beam objects"""
 
     def __init__(self, beams: Iterable[Beam] = None, parent=None):
-
         super().__init__(parent)
         beams = [] if beams is None else beams
         self._beams = sorted(beams, key=attrgetter("name"))
@@ -180,7 +182,7 @@ class Beams(BaseCollections):
             raise NameIsNoneError("Name is not allowed to be None.")
 
         if beam.name in self._dmap.keys():
-            logging.warning(f'Beam with name "{beam.name}" already exists. Will not add')
+            logger.warning(f'Beam with name "{beam.name}" already exists. Will not add')
             return self._dmap[beam.name]
 
         self._dmap[beam.guid] = beam
@@ -392,7 +394,7 @@ class Connections(BaseCollections):
     def get_from_name(self, name: str):
         result = self._dmap.get(name, None)
         if result is None:
-            logging.error(f'No Joint with the name "{name}" found within this connection object')
+            logger.error(f'No Joint with the name "{name}" found within this connection object')
         return result
 
     def add(self, joint: JointBase, point_tol=Settings.point_tol):
@@ -473,34 +475,34 @@ class NumericMapped(BaseCollections):
 class Materials(NumericMapped):
     """Collection of materials"""
 
-    def __init__(self, materials: Iterable[Material] = None, parent: Union[Part, Assembly] = None, units="m"):
+    def __init__(self, materials: Iterable[Material] = None, parent: Union[Part, Assembly] = None, units=Units.M):
         super().__init__(parent)
-        self._materials = sorted(materials, key=attrgetter("name")) if materials is not None else []
-        self.recreate_name_and_id_maps(self._materials)
+        self.materials = sorted(materials, key=attrgetter("name")) if materials is not None else []
+        self.recreate_name_and_id_maps(self.materials)
         self._units = units
 
     def __contains__(self, item: Material):
         return item.name in self._name_map.keys()
 
     def __len__(self) -> int:
-        return len(self._materials)
+        return len(self.materials)
 
     def __iter__(self) -> Iterable[Material]:
-        return iter(self._materials)
+        return iter(self.materials)
 
     def __getitem__(self, index):
-        result = self._materials[index]
+        result = self.materials[index]
         return Materials(result) if isinstance(index, slice) else result
 
     def __eq__(self, other: Materials):
         if not isinstance(other, Materials):
             return NotImplemented
-        return self._materials == other._materials
+        return self.materials == other.materials
 
     def __ne__(self, other: Materials):
         if not isinstance(other, Materials):
             return NotImplemented
-        return self._materials != other._materials
+        return self.materials != other.materials
 
     def __add__(self, other: Materials):
         if self.parent is None:
@@ -514,13 +516,13 @@ class Materials(NumericMapped):
         rpr = reprlib.Repr()
         rpr.maxlist = 8
         rpr.maxlevel = 1
-        return f"Materials({rpr.repr(self._materials) if self._materials else ''})"
+        return f"Materials({rpr.repr(self.materials) if self.materials else ''})"
 
     def merge_materials_by_properties(self):
         models = []
 
         final_mats = []
-        for i, mat in enumerate(self._materials):
+        for i, mat in enumerate(self.materials):
             if mat.model.unique_props() not in models:
                 models.append(mat.model.unique_props())
                 final_mats.append(mat)
@@ -530,11 +532,11 @@ class Materials(NumericMapped):
                 for ref in mat.refs:
                     ref.material = replacement_mat
 
-        self._materials = final_mats
-        self.recreate_name_and_id_maps(self._materials)
+        self.materials = final_mats
+        self.recreate_name_and_id_maps(self.materials)
 
     def index(self, item: Material):
-        return self._materials.index(item)
+        return self.materials.index(item)
 
     def count(self, item: Material):
         return int(item in self)
@@ -556,7 +558,7 @@ class Materials(NumericMapped):
         for mat_id in sorted(self.id_map.keys()):
             mat = self.get_by_id(mat_id)
             mat.id = next(cnt)
-        self.recreate_name_and_id_maps(self._materials)
+        self.recreate_name_and_id_maps(self.materials)
 
     @property
     def name_map(self) -> Dict[str, Material]:
@@ -572,26 +574,33 @@ class Materials(NumericMapped):
 
     @units.setter
     def units(self, value):
+        if isinstance(value, str):
+            value = Units.from_str(value)
         if value != self._units:
-            for m in self._materials:
+            for m in self.materials:
                 m.units = value
             self._units = value
 
     def add(self, material) -> Material:
         if material in self:
-            return self._name_map[material.name]
+            existing_mat = self._name_map[material.name]
+            for elem in material.refs:
+                if elem not in existing_mat.refs:
+                    existing_mat.refs.append(elem)
+            return existing_mat
 
         if material.id is None or material.id in self._id_map.keys():
-            material.id = len(self._materials) + 1
+            material.id = len(self.materials) + 1
+
         self._id_map[material.id] = material
         self._name_map[material.name] = material
-        self._materials.append(material)
+        self.materials.append(material)
 
         return material
 
 
 class Sections(NumericMapped):
-    def __init__(self, sections: Iterable[Section] = None, parent: Union[Part, Assembly] = None, units="m"):
+    def __init__(self, sections: Iterable[Section] = None, parent: Part | Assembly = None, units=Units.M):
         sec_id = Counter(1)
         super(Sections, self).__init__(parent=parent)
         sections = [] if sections is None else sections
@@ -608,7 +617,12 @@ class Sections(NumericMapped):
         self.recreate_name_and_id_maps(self._sections)
 
         if len(self._name_map.keys()) != len(self._id_map.keys()):
-            logging.warning(f"Non-unique ids or name for section container belonging to part '{parent}'")
+            import collections
+
+            names = [sec.name for sec in self._sections]
+            counts = collections.Counter(names)
+            filtered_elements = {element: count for element, count in counts.items() if count > 1}
+            logger.warning(f"The following sections are non-unique '{filtered_elements}'")
 
     def renumber_id(self, start_id=1):
         cnt = Counter(start=start_id)
@@ -632,7 +646,7 @@ class Sections(NumericMapped):
 
     def __add__(self, other: Sections):
         if self.parent is None:
-            logging.error(f'Parent is None for Sections container "{self}"')
+            logger.error(f'Parent is None for Sections container "{self}"')
         for sec in other:
             sec.parent = self.parent
         other.renumber_id(self.max_id + 1)
@@ -690,7 +704,6 @@ class Sections(NumericMapped):
         return self._name_map
 
     def add(self, section: Section) -> Section:
-
         if section.name is None:
             raise Exception("Name is not allowed to be None.")
 
@@ -700,11 +713,21 @@ class Sections(NumericMapped):
 
         if section in self._sections:
             index = self._sections.index(section)
-            return self._sections[index]
+            existing_section = self._sections[index]
+            for elem in section.refs:
+                elem.section = existing_section
+                if elem not in existing_section.refs:
+                    existing_section.refs.append(elem)
+            return existing_section
 
         if section.name in self._name_map.keys():
-            logging.info(f'Section with same name "{section.name}" already exists. Will use that section instead')
-            return self._name_map[section.name]
+            logger.info(f'Section with same name "{section.name}" already exists. Will use that section instead')
+            existing_section = self._name_map[section.name]
+            for elem in section.refs:
+                elem.section = existing_section
+                if elem not in existing_section.refs:
+                    existing_section.refs.append(elem)
+            return existing_section
 
         if section.id is None:
             section.id = self.max_id + 1
@@ -719,7 +742,7 @@ class Sections(NumericMapped):
         return section
 
     @property
-    def sections(self) -> List[Section]:
+    def sections(self) -> list[Section]:
         return self._sections
 
     @property
@@ -728,6 +751,8 @@ class Sections(NumericMapped):
 
     @units.setter
     def units(self, value):
+        if isinstance(value, str):
+            value = Units.from_str(value)
         if value != self._units:
             for m in self._sections:
                 m.units = value
@@ -795,6 +820,15 @@ class Nodes:
         else:
             return np.array([n.p for n in self._nodes])
 
+    def to_fem_nodes(self) -> FemNodes:
+        from ada.fem.results.common import FemNodes
+
+        node_refs = self.to_np_array(include_id=True)
+        identifiers = node_refs[:, 0]
+        coords = node_refs[:, 1:]
+
+        return FemNodes(coords, identifiers)
+
     def __contains__(self, item):
         return item in self._nodes
 
@@ -979,7 +1013,7 @@ class Nodes:
                 nearest_node = res[0]
                 vlen = vector_length(nearest_node.p - node.p)
                 if vlen < point_tol:
-                    logging.debug(f'Replaced new node with node id "{nearest_node.id}" found within point tolerances')
+                    logger.debug(f'Replaced new node with node id "{nearest_node.id}" found within point tolerances')
                     return nearest_node
 
         insert_node(node, index)
@@ -994,11 +1028,11 @@ class Nodes:
         nodes = list(nodes) if isinstance(nodes, Iterable) else [nodes]
         for node in nodes:
             if node in self._nodes:
-                logging.debug(f"Removing {node}")
+                logger.debug(f"Removing {node}")
                 self._nodes.pop(self._nodes.index(node))
                 self.renumber()
             else:
-                logging.error(f"'{node}' not found in node-container.")
+                logger.error(f"'{node}' not found in node-container.")
 
     def remove_standalones(self) -> None:
         """Remove nodes that are without any usage references"""
