@@ -1,16 +1,17 @@
 from __future__ import annotations
 
-import logging
-from typing import TYPE_CHECKING, Iterable, List, Optional, Union
+from enum import Enum
+from typing import TYPE_CHECKING, Callable, Iterable, List, Optional, Union
 
 import numpy as np
 
 from ada.base.physical_objects import BackendGeom
+from ada.base.units import Units
 from ada.concepts.bounding_box import BoundingBox
 from ada.concepts.curves import CurvePoly, CurveRevolve
 from ada.concepts.points import Node, get_singular_node_by_volume
 from ada.concepts.transforms import Placement
-from ada.config import Settings
+from ada.config import Settings, get_logger
 from ada.core.utils import Counter, roundoff
 from ada.core.vector_utils import (
     angle_between,
@@ -30,15 +31,16 @@ if TYPE_CHECKING:
     from OCC.Core.TopoDS import TopoDS_Shape
 
     from ada.concepts.connections import JointBase
-    from ada.concepts.levels import Part
+    from ada.concepts.spatial import Part
     from ada.fem.elements import HingeProp
-    from ada.ifc.concepts import IfcRef
+    from ada.ifc.store import IfcStore
 
 section_counter = Counter(1)
 material_counter = Counter(1)
+logger = get_logger()
 
 
-class Justification:
+class Justification(Enum):
     NA = "neutral axis"
     TOS = "top of steel"
 
@@ -62,35 +64,33 @@ class Beam(BackendGeom):
     def __init__(
         self,
         name,
-        n1: Union[Node, Iterable] = None,
-        n2: Union[Node, Iterable] = None,
-        sec: Union[str, Section] = None,
-        mat: Union[str, Material] = None,
-        tap: Union[str, Section] = None,
+        n1: Node | Iterable = None,
+        n2: Node | Iterable = None,
+        sec: str | Section = None,
+        mat: str | Material = None,
+        tap: str | Section = None,
         jusl=JUSL_TYPES.NA,
         up=None,
         angle=0.0,
-        curve: Union[CurvePoly, CurveRevolve] = None,
+        curve: CurvePoly | CurveRevolve = None,
         e1=None,
         e2=None,
         colour=None,
         parent: Part = None,
         metadata=None,
         opacity=1.0,
-        units="m",
-        ifc_elem=None,
+        units=Units.M,
         guid=None,
         placement=Placement(),
-        ifc_ref: IfcRef = None,
+        ifc_store: IfcStore = None,
     ):
         super().__init__(
             name,
             metadata=metadata,
             units=units,
             guid=guid,
-            ifc_elem=ifc_elem,
             placement=placement,
-            ifc_ref=ifc_ref,
+            ifc_store=ifc_store,
             colour=colour,
             opacity=opacity,
         )
@@ -124,13 +124,15 @@ class Beam(BackendGeom):
         # Section and Material setup
         self._section, self._taper = get_section(sec)
         self._section.refs.append(self)
-        self._taper.refs.append(self)
+
         self._material = get_material(mat)
         self._material.refs.append(self)
 
         if tap is not None:
             self._taper, _ = get_section(tap)
 
+        self._taper.refs.append(self)
+
         self._section.parent = self
         self._taper.parent = self
 
@@ -138,6 +140,16 @@ class Beam(BackendGeom):
         self._init_orientation(angle, up)
         self.add_beam_to_node_refs()
 
+    @staticmethod
+    def from_list_of_coords(
+        list_of_coords: list[tuple], sec: Section | str, mat: Material | str = None, name_gen: Callable = None
+    ) -> list[Beam]:
+        beams = []
+        ngen = name_gen if name_gen is not None else Counter(prefix="bm")
+        for p1, p2 in zip(list_of_coords[:-1], list_of_coords[1:]):
+            beams.append(Beam(next(ngen), p1, p2, sec, mat))
+        return beams
+
     def _init_orientation(self, angle=None, up=None) -> None:
         xvec = unit_vector(self.n2.p - self.n1.p)
         tol = 1e-3
@@ -200,7 +212,7 @@ class Beam(BackendGeom):
             length_fraction = length / self.length
             splitting_node = self.get_node_on_beam_by_fraction(length_fraction)
         else:
-            logging.warning(f"Beam {self} is not split as inconclusive info is provided.")
+            logger.warning(f"Beam {self} is not split as inconclusive info is provided.")
             return None
 
         node_on_beam = self.parent.fem.nodes.add(splitting_node)
@@ -277,11 +289,6 @@ class Beam(BackendGeom):
 
         return nodes_p1, nodes_p2
 
-    def _generate_ifc_elem(self):
-        from ada.ifc.write.write_beams import write_ifc_beam
-
-        return write_ifc_beam(self)
-
     def calc_con_points(self, point_tol=Settings.point_tol):
         from ada.core.vector_utils import sort_points_by_dist
 
@@ -307,7 +314,7 @@ class Beam(BackendGeom):
                     if m != self:
                         is_ecc, end = is_mem_eccentric(m, con.centre)
                         if is_ecc:
-                            logging.info(f'do something with end "{end}"')
+                            logger.info(f'do something with end "{end}"')
                             points.append(tuple(end))
 
         midpoints = []
@@ -341,20 +348,27 @@ class Beam(BackendGeom):
 
         return midpoints
 
+    def copy_to(self, p1, p2, name: str) -> Beam:
+        return Beam(name, p1, p2, sec=self.section, tap=self.taper, mat=self.material)
+
     @property
     def units(self):
         return self._units
 
     @units.setter
     def units(self, value):
-        if self._units != value:
-            self.n1.units = value
-            self.n2.units = value
-            self.section.units = value
-            self.material.units = value
-            for pen in self.penetrations:
-                pen.units = value
-            self._units = value
+        if isinstance(value, str):
+            value = Units.from_str(value)
+        if self._units == value:
+            return
+
+        self.n1.units = value
+        self.n2.units = value
+        self.section.units = value
+        self.material.units = value
+        for pen in self.penetrations:
+            pen.units = value
+        self._units = value
 
     @property
     def section(self) -> Section:
@@ -362,16 +376,7 @@ class Beam(BackendGeom):
 
     @section.setter
     def section(self, value: Section):
-        section = self.parent.add_section(value)
-
-        if self.taper == self.section:
-            self.taper = section
-
-        if self in self.section.refs:
-            self.section.refs.remove(self)
-
-        self._section = section
-        self._section.refs.append(self)
+        self._section = value
 
     @property
     def taper(self) -> Section:
@@ -379,11 +384,7 @@ class Beam(BackendGeom):
 
     @taper.setter
     def taper(self, value: Section):
-        if self in self.taper.refs:
-            self.taper.refs.remove(self)
-
         self._taper = value
-        self._taper.refs.append(self)
 
     @property
     def material(self) -> Material:
@@ -625,7 +626,7 @@ class Beam(BackendGeom):
             try:
                 res = oval != val
             except ValueError as e:
-                logging.error(e)
+                logger.error(e)
                 return True
 
             if res is True:
