@@ -6,6 +6,7 @@ import numpy as np
 
 from ada import Part
 from ada.base.physical_objects import BackendGeom
+from ada.base.units import Units
 from ada.concepts.curves import CurvePoly
 from ada.concepts.primitives import PrimBox
 from ada.concepts.transforms import Placement
@@ -43,14 +44,11 @@ class Wall(BackendGeom):
         offset=TYPES_JUSL.CENTER,
         metadata=None,
         colour=None,
-        ifc_elem=None,
-        units="m",
+        units=Units.M,
         guid=None,
         opacity=1.0,
     ):
-        super().__init__(
-            name, guid=guid, metadata=metadata, units=units, ifc_elem=ifc_elem, colour=colour, opacity=opacity
-        )
+        super().__init__(name, guid=guid, metadata=metadata, units=units, colour=colour, opacity=opacity)
 
         self._name = name
         self.placement = placement
@@ -236,10 +234,10 @@ class Wall(BackendGeom):
 
     @property
     def openings_extrusions(self):
-        from ada.concepts.levels import Part
+        from ada.concepts.spatial import Part
 
         op_extrudes = []
-        if self.units == "m":
+        if self.units == Units.M:
             tol = 0.4
         else:
             tol = 400
@@ -279,10 +277,10 @@ class Wall(BackendGeom):
 
     @units.setter
     def units(self, value):
+        if isinstance(value, str):
+            value = Units.from_str(value)
         if value != self._units:
-            from ada.core.utils import unit_length_conversion
-
-            scale_factor = unit_length_conversion(self._units, value)
+            scale_factor = Units.get_scale_factor(self._units, value)
             self._height *= scale_factor
             self._thickness *= scale_factor
             self._offset *= scale_factor
