@@ -1,11 +1,11 @@
 from __future__ import annotations
 
-import logging
 from dataclasses import dataclass, field
 from itertools import chain
-from typing import TYPE_CHECKING, Dict, List, Tuple, Union
+from typing import TYPE_CHECKING, Dict, Iterable, List, Tuple, Union
 
 from ada.concepts.containers import Nodes
+from ada.config import get_logger
 
 from .containers import FemElements, FemSections, FemSets
 from .sets import FemSet
@@ -25,6 +25,7 @@ if TYPE_CHECKING:
         FemSection,
         Interaction,
         InteractionProperty,
+        Load,
         Mass,
         PredefinedField,
         Spring,
@@ -34,8 +35,11 @@ if TYPE_CHECKING:
         StepSteadyState,
         Surface,
     )
+    from ada.fem.results.common import Mesh
+    from ada.fem.steps import Step
 
 _step_types = Union["StepSteadyState", "StepEigen", "StepImplicit", "StepExplicit"]
+logger = get_logger()
 
 
 @dataclass
@@ -99,7 +103,7 @@ class FEM:
             else:
                 fs = self.sets.add(section.elset)
             if fs != section.elset:
-                logging.info(f'Element set "{section.elset}" is replaced by {fs}')
+                logger.info(f'Element set "{section.elset}" is replaced by {fs}')
                 section.elset = fs
         if section.material.parent is None and self.parent is not None:
             self.parent.add_material(section.material)
@@ -112,7 +116,7 @@ class FEM:
 
         bc.parent = self
         if bc.fem_set.parent is None:
-            logging.debug("Bc FemSet has no parent. Adding to self")
+            logger.debug("Bc FemSet has no parent. Adding to self")
             self.sets.add(bc.fem_set)
 
         self.bcs.append(bc)
@@ -314,23 +318,49 @@ class FEM:
 
         return True
 
-    def get_all_bcs(self):
+    def get_all_steps(self) -> list[Step]:
+        assembly = self.parent.get_assembly()
+        steps = []
+        for p in assembly.get_all_parts_in_assembly(include_self=True):
+            if len(p.fem.steps) == 0:
+                continue
+            steps += p.fem.steps
+        return steps
+
+    def get_all_bcs(self) -> Iterable[Bc]:
         """Get all the boundary conditions in the entire assembly"""
         assembly = self.parent.get_assembly()
         return chain.from_iterable(
             (
-                [bc for bc in assembly.fem.bcs],
+                assembly.fem.bcs,
                 [bc for p in assembly.get_all_parts_in_assembly() for bc in p.fem.bcs],
             )
         )
 
-    def get_all_loads(self):
+    def get_all_masses(self) -> Iterable[Mass]:
+        """Get all the Masses in the entire assembly"""
+        assembly = self.parent.get_assembly()
+        return chain.from_iterable(
+            (
+                assembly.fem.masses.values(),
+                [mass for p in assembly.get_all_parts_in_assembly() for mass in p.fem.masses.values()],
+            )
+        )
+
+    def get_all_loads(self) -> list[Load]:
         loads = []
         for step in self.steps:
             for load in step.loads:
                 loads.append(load)
         return loads
 
+    def to_mesh(self) -> Mesh:
+        from ada.fem.results.common import Mesh
+
+        fem_nodes = self.nodes.to_fem_nodes()
+        elem_blocks = self.elements.to_elem_blocks()
+        return Mesh(elem_blocks, fem_nodes)
+
     @property
     def instance_name(self):
         return self.name if self.name is not None else f"{self.parent.name}-1"
