@@ -1,8 +1,5 @@
 from dataclasses import dataclass
 
-from ada.fem.formats import FEATypes
-from ada.fem.steps import SolverOptions
-
 
 class StabilizeTypes:
     ENERGY = "energy"
@@ -34,8 +31,7 @@ class Stabilize:
 
 
 @dataclass
-class SolverOptionsAbaqus(SolverOptions):
-
+class AbaqusStepOptions:
     init_accel_calc: bool = True
     restart_int: int = None
     unsymm: bool = False
@@ -45,8 +41,5 @@ class SolverOptionsAbaqus(SolverOptions):
     :param init_accel_calc: Calculate Initial acceleration in the beginning of the step
     :param restart_int: Restart interval
     :param unsymm: Unsymmetric Matrix storage (default=False)
-    :param stabilize: Default=None.
+    :param stabilize: Default=None. Abaqus CAE defaults to stabilize=0.0002 and allsdtol=0.05
     """
-
-    def __post_init__(self):
-        self.solver = FEATypes.ABAQUS
