@@ -1,13 +1,14 @@
-import logging
 from typing import TYPE_CHECKING
 
+from ada.config import get_logger
 from ada.fem import FemSection
 from ada.fem.steps import StepExplicit
-from ada.sections import GeneralProperties, Section, SectionCat
+from ada.sections import GeneralProperties, Section
 
 if TYPE_CHECKING:
     from ada import FEM
 
+logger = get_logger()
 log_fin = "Please check your result and input. This is not a validated method of solving this issue"
 
 
@@ -83,7 +84,7 @@ def line_section_props(fem_sec: FemSection):
             #     sec.w_btn = new_width
             # else:
             #     sec.w_top = new_width
-            logging.info(f"For {fem_sec.name}: t_fbtn + t_w > min(w_top, w_btn). {log_fin}")
+            logger.info(f"For {fem_sec.name}: t_fbtn + t_w > min(w_top, w_btn). {log_fin}")
         return f"{sec.h / 2}, {sec.h}, {sec.w_btn}, {sec.w_top}, {sec.t_fbtn}, {sec.t_ftop}, {sec.t_w}\n {n1}"
     elif sec_data == "BOX":
         if sec.t_w * 2 > min(sec.w_top, sec.w_btn):
@@ -110,7 +111,7 @@ def line_cross_sec_type_str(fem_sec: FemSection):
     from ada.sections.categories import BaseTypes
 
     bt = BaseTypes
-    base_type = SectionCat.get_shape_type(fem_sec.section)
+
     sec_map = {
         bt.CIRCULAR: "CIRC",
         bt.IPROFILE: "I",
@@ -121,12 +122,12 @@ def line_cross_sec_type_str(fem_sec: FemSection):
         bt.CHANNEL: "GENERAL",
         bt.FLATBAR: "RECT",
     }
-    sec_str = sec_map.get(base_type, None)
+    sec_str = sec_map.get(fem_sec.section.type, None)
     if sec_str is None:
         raise Exception(f'Section type "{sec_type}" is not added to Abaqus beam export yet')
 
-    if base_type in [bt.CHANNEL]:
-        logging.error(f'Profile type "{sec_type}" is not supported by Abaqus. Using a General Section instead')
+    if fem_sec.section.type in [bt.CHANNEL]:
+        logger.error(f'Profile type "{sec_type}" is not supported by Abaqus. Using a General Section instead')
 
     return sec_str
 
@@ -138,23 +139,23 @@ def line_temperature_str(fem_sec: FemSection):
 
 def eval_general_properties(section: Section) -> GeneralProperties:
     gp = section.properties
-    name = gp.parent.parent.name
+    name = section.name
     if gp.Ix <= 0.0:
         gp.Ix = 1
-        logging.error(f"Section {name} Ix <= 0.0. Changing to 2. {log_fin}")
+        logger.warning(f"Section {name} Ix <= 0.0. Changing to 2. {log_fin}")
     if gp.Iy <= 0.0:
         gp.Iy = 2
-        logging.error(f"Section {name} Iy <= 0.0. Changing to 2. {log_fin}")
+        logger.warning(f"Section {name} Iy <= 0.0. Changing to 2. {log_fin}")
     if gp.Iz <= 0.0:
         gp.Iz = 2
-        logging.error(f"Section {name} Iz <= 0.0. Changing to 2. {log_fin}")
+        logger.warning(f"Section {name} Iz <= 0.0. Changing to 2. {log_fin}")
     if gp.Iyz <= 0.0:
         gp.Iyz = (gp.Iy + gp.Iz) / 2
-        logging.error(f"Section {name} Iyz <= 0.0. Changing to (Iy + Iz) / 2. {log_fin}")
+        logger.warning(f"Section {name} Iyz <= 0.0. Changing to (Iy + Iz) / 2. {log_fin}")
     if gp.Iy * gp.Iz - gp.Iyz**2 < 0:
         old_y = str(gp.Iy)
         gp.Iy = 1.1 * (gp.Iy + (gp.Iyz**2) / gp.Iz)
-        logging.error(
+        logger.warning(
             f"Warning! Section {name}: I(11)*I(22)-I(12)**2 MUST BE POSITIVE. " f"Mod Iy={old_y} to {gp.Iy}. {log_fin}"
         )
     if (-(gp.Iy + gp.Iz) / 2 < gp.Iyz <= (gp.Iy + gp.Iz) / 2) is False:
