@@ -1,6 +1,5 @@
 from __future__ import annotations
 
-import logging
 from typing import TYPE_CHECKING, List, Tuple, Union
 
 import ifcopenshell
@@ -11,31 +10,57 @@ from ifcopenshell.util.unit import get_prefix_multiplier
 
 import ada.core.constants as ifco
 from ada.concepts.transforms import Transform
-from ada.config import Settings
+from ada.config import get_logger
 from ada.core.file_system import get_list_of_files
-from ada.core.utils import roundoff
 
 if TYPE_CHECKING:
     from ada import Assembly, Beam
 
+logger = get_logger()
+
+
+def create_reference_subrep(f, global_axes):
+    model_rep = f.create_entity("IfcGeometricRepresentationContext", None, "Model", 3, 1.0e-05, global_axes, None)
+    body_sub_rep = f.create_entity(
+        "IfcGeometricRepresentationSubContext",
+        "Body",
+        "Model",
+        None,
+        None,
+        None,
+        None,
+        model_rep,
+        None,
+        "MODEL_VIEW",
+        None,
+    )
+    ref_sub_rep = f.create_entity(
+        "IfcGeometricRepresentationSubContext",
+        "Reference",
+        "Model",
+        None,
+        None,
+        None,
+        None,
+        model_rep,
+        None,
+        "GRAPH_VIEW",
+        None,
+    )
+
+    return {"model": model_rep, "body": body_sub_rep, "reference": ref_sub_rep}
+
 
 def ifc_dir(f: ifcopenshell.file, vec: Tuple[float, float, float]):
     return f.create_entity("IfcDirection", to_real(vec))
 
 
-def get_tolerance(units):
-    tol_map = dict(m=Settings.mtol, mm=Settings.mmtol)
-    if units not in tol_map.keys():
-        raise ValueError(f'Unrecognized unit "{units}"')
-    return tol_map[units]
-
-
 def ensure_guid_consistency(a: Assembly, project_prefix):
     """Function to edit the global IDs of your elements when they are arbitrarily created from upstream data dump"""
+    a.ifc_store.sync()
     ensure_uniqueness = dict()
     for p in a.get_all_parts_in_assembly():
         p.guid = create_guid(project_prefix + p.name)
-
         if p.guid in ensure_uniqueness.keys():
             # p_other = ensure_uniqueness.get(p.guid)
             # ancestors1 = p.get_ancestors()
@@ -94,9 +119,9 @@ def create_ifc_placement(f: ifcopenshell.file, origin=ifco.O, loc_z=ifco.Z, loc_
     :return:
     """
 
-    ifc_loc_z = f.createIfcDirection(to_real(loc_z))
-    ifc_loc_x = f.createIfcDirection(to_real(loc_x))
-    return f.createIfcAxis2Placement3D(ifc_p(f, origin), ifc_loc_z, ifc_loc_x)
+    ifc_loc_z = f.create_entity("IfcDirection", to_real(loc_z))
+    ifc_loc_x = f.create_entity("IfcDirection", to_real(loc_x))
+    return f.create_entity("IfcAxis2Placement3D", ifc_p(f, origin), ifc_loc_z, ifc_loc_x)
 
 
 def create_local_placement(f: ifcopenshell.file, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relative_to=None):
@@ -157,18 +182,19 @@ def generate_tpl_ifc_file(file_name, project, schema, units, user):
     :type user: ada.config.User
     :return:
     """
-
     import time
 
+    from ada.base.units import Units
+
     from .ifc_template import tpl_create
 
     timestamp = time.time()
     timestring = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime(timestamp))
     application, application_version = "IfcOpenShell", "0.6"
     project_globalid = create_guid()
-    if units == "m":
+    if units == Units.M:
         units_str = "$,.METRE."
-    elif units == "mm":
+    elif units == Units.MM:
         units_str = ".MILLI.,.METRE."
     else:
         raise ValueError(f'Unrecognized unit prefix "{units}"')
@@ -207,6 +233,11 @@ def create_ifcpolyline(ifcfile, point_list):
     return polyline
 
 
+def create_axis(f, points, context):
+    polyline = create_ifcpolyline(f, points)
+    return f.createIfcShapeRepresentation(context, "Axis", "Curve3D", [polyline])
+
+
 def create_ifcindexpolyline(ifcfile, points, seg_index):
     """
     Assumes a point list whereas all points that are to be used for creating arc-segments will have 4 values
@@ -321,50 +352,47 @@ def create_ifcrightcylinder(ifc_file, ifcaxis2placement, height, radius):
     return ifcextrudedareasolid
 
 
-def create_property_set(name, ifc_file, metadata_props, owner_history):
+def ifc_value_map(f, value):
+    value_map = {str: "IfcText", float: "IfcReal", int: "IfcInteger", bool: "IfcBoolean"}
+    if type(value) in (np.float64,):
+        value = float(value)
+    ifc_type = value_map.get(type(value), None)
+    if ifc_type is None:
+        logger.warning(f'Unable to find suitable IFC type for "{type(value)}". Will convert it to string')
+        return f.create_entity("IfcText", str(value))
 
+    return f.create_entity(ifc_type, value)
+
+
+def ifc_list_value(f, name: str, list_value: list, owner_history):
+    list_values = []
+    for x in list_value:
+        if isinstance(x, (list, tuple, np.ndarray)):
+            list_values.append(ifc_list_value(f, f"{name}_sub", x, owner_history))
+        elif isinstance(x, dict):
+            list_values.append(create_property_set(f"{name}_sub", f, x, owner_history))
+        else:
+            list_values.append(ifc_value_map(f, x))
+
+    return f.create_entity("IfcPropertyListValue", Name=name, ListValues=list_values)
+
+
+def create_property_set(name, ifc_file, metadata_props, owner_history):
     properties = []
 
-    def ifc_value(v_):
-        return ifc_file.create_entity("IfcText", str(v_))
-
-    def to_str(in_enum):
-        return (
-            f"{in_enum}".replace("(", "")
-            .replace(")", "")
-            .replace(" ", "")
-            .replace(",", ";")
-            .replace("[", "")
-            .replace("]", "")
-        )
-
     for key, value in metadata_props.items():
-        if type(value) in (tuple, list):
-            if type(value[0]) in (list, tuple, np.ndarray):
-                for i, v in enumerate(value):
-                    if type(v) is np.ndarray:
-                        v = [roundoff(x) for x in v]
-                    properties.append(
-                        ifc_file.create_entity(
-                            "IfcPropertySingleValue",
-                            Name=f"{key}_{i}",
-                            NominalValue=ifc_value(to_str(v)),
-                        )
-                    )
-            else:
-                properties.append(
-                    ifc_file.create_entity(
-                        "IfcPropertySingleValue",
-                        Name=key,
-                        NominalValue=ifc_value(to_str(value)),
-                    )
-                )
+        if isinstance(value, (list, tuple, np.ndarray)):
+            properties.append(ifc_list_value(ifc_file, key, value, owner_history))
+        elif isinstance(value, dict):
+            if len(value.keys()) == 0:
+                continue
+            properties.append(create_property_set(f"{name}_sub", ifc_file, value, owner_history))
         else:
             properties.append(
                 ifc_file.create_entity(
                     "IfcPropertySingleValue",
                     Name=key,
-                    NominalValue=ifc_value(value),
+                    NominalValue=ifc_value_map(ifc_file, value),
                 )
             )
 
@@ -379,7 +407,7 @@ def create_property_set(name, ifc_file, metadata_props, owner_history):
 
 
 def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props, owner_history):
-    logging.info(f'Adding "{name}" properties to IFC Element "{ifc_elem}"')
+    logger.info(f'Adding "{name}" properties to IFC Element "{ifc_elem}"')
 
     props = create_property_set(name, ifc_file, elem_props, owner_history=owner_history)
     ifc_file.createIfcRelDefinesByProperties(
@@ -392,16 +420,18 @@ def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props, owner_history):
     )
 
 
-def add_multiple_props_to_elem(metadata_props, elem, f, owner_history):
-    if len(metadata_props.keys()) > 0:
-        if type(list(metadata_props.values())[0]) is dict:
-            for pro_id, prop_ in metadata_props.items():
-                add_properties_to_elem(pro_id, f, elem, prop_, owner_history=owner_history)
-        else:
-            add_properties_to_elem("Properties", f, elem, metadata_props, owner_history=owner_history)
+def write_elem_property_sets(metadata_props, elem, f, owner_history) -> None:
+    if len(metadata_props.keys()) == 0:
+        return None
+
+    if type(list(metadata_props.values())[0]) is dict:
+        for pro_id, prop_ in metadata_props.items():
+            add_properties_to_elem(pro_id, f, elem, prop_, owner_history=owner_history)
+    else:
+        add_properties_to_elem("Properties", f, elem, metadata_props, owner_history=owner_history)
 
 
-def to_real(v) -> Union[float, List[float]]:
+def to_real(v) -> float | list[float]:
     from ada import Node
 
     if type(v) is float:
@@ -420,18 +450,6 @@ def to_real(v) -> Union[float, List[float]]:
 
 
 def add_negative_extrusion(f, origin, loc_z, loc_x, depth, points, parent):
-    """
-
-    :param f:
-    :param origin:
-    :param loc_z:
-    :param loc_x:
-    :param depth:
-    :param points:
-    :param parent:
-    :return:
-    """
-
     context = f.by_type("IfcGeometricRepresentationContext")[0]
     owner_history = f.by_type("IfcOwnerHistory")[0]
 
@@ -502,12 +520,14 @@ def calculate_unit_scale(file):
     return unit_scale
 
 
-def get_unit_type(file):
+def get_unit_type(file: ifcopenshell.file):
+    from ada.base.units import Units
+
     value = calculate_unit_scale(file)
     if value == 0.001:
-        return "mm"
+        return Units.MM
     elif value == 1:
-        return "m"
+        return Units.M
     else:
         raise NotImplementedError(f'Unit scale of "{value}" is not yet supported')
 
@@ -581,7 +601,6 @@ def scale_ifc_file_object(ifc_file, scale_factor):
     for ifc_class, attributes in classes_to_modify.items():
         for element in ifc_file.by_type(ifc_class):
             for attribute in attributes:
-
                 old_val = getattr(element, attribute)
                 if old_val is None:
                     continue
@@ -638,7 +657,7 @@ def merge_ifc_files(parent_dir, output_file_name, clean_files=False, include_ele
     f = ifcopenshell.open(files[0])
     for i, fp in enumerate(files[1:]):
         checkpoint = time.time()
-        print(f'merging products ({i+1} of {len(files)-1}) from "{fp}"')
+        print(f'merging products ({i + 1} of {len(files) - 1}) from "{fp}"')
         fn = ifcopenshell.open(fp)
         print(f"file opened in {time.time() - checkpoint:.2f} seconds")
         checkpoint = time.time()
@@ -689,7 +708,7 @@ def convert_bm_jusl_to_ifc(bm: "Beam") -> int:
 
     if jusl_val is None:
         if jusl != jt.NA:
-            logging.error(f'Unknown JUSL value "{jusl}". Using NA')
+            logger.error(f'Unknown JUSL value "{jusl}". Using NA')
         return 5
 
     return jusl_val
@@ -699,7 +718,7 @@ def scale_ifc_file(current_ifc, new_ifc):
     oval = calculate_unit_scale(current_ifc)
     nval = calculate_unit_scale(new_ifc)
     if oval != nval:
-        logging.error("Running Unit Conversion on IFC import. This is still highly unstable")
+        logger.error("Running Unit Conversion on IFC import. This is still highly unstable")
         # length_unit = f.createIfcSIUnit(None, "LENGTHUNIT", None, "METRE")
         # unit_assignment = f.createIfcUnitAssignment((length_unit,))
         new_file = scale_ifc_file_object(new_ifc, nval)
@@ -711,7 +730,7 @@ def tesselate_shape(shape, schema, tol):
     serialized_geom = ifcopenshell.geom.serialise(schema, occ_string)
 
     if serialized_geom is None:
-        logging.debug("Starting serialization of geometry")
+        logger.debug("Starting serialization of geometry")
         serialized_geom = ifcopenshell.geom.tesselate(schema, occ_string, tol)
 
     return serialized_geom
@@ -749,23 +768,12 @@ def get_all_subtypes(entity: ifcopenshell.ifcopenshell_wrapper.entity, subtypes=
     return subtypes
 
 
-def get_geom_items(ifc_schema):
-    wrap = ifcopenshell.ifcopenshell_wrapper
-    schema = wrap.schema_by_name(ifc_schema)
-    all_entities = {x.name(): x for x in schema.declarations()}
-    entity: wrap.entity = all_entities["IfcGeometricRepresentationItem"]
-    subtypes = get_all_subtypes(entity)
-
-    return subtypes
-
-
-_geom_items = None
-
-
 def get_representation_items(f: ifcopenshell.file, ifc_elem: ifcopenshell.entity_instance):
     geom_items = [
         "IfcTriangulatedFaceSet",
         "IfcExtrudedAreaSolid",
+        "IfcExtrudedAreaSolid",
+        "IfcExtrudedAreaSolidTapered",
         "IfcRevolvedAreaSolid",
         "IfcClosedShell",
     ]
@@ -776,3 +784,58 @@ def get_representation_items(f: ifcopenshell.file, ifc_elem: ifcopenshell.entity
             f.traverse(ifc_elem),
         )
     )
+
+
+def ifc_file_to_stp(ifc_file, stp_file, step_schema="AP242", step_assembly_mode=1):
+    """Convert an IFC file to STEP file"""
+
+    import os
+    import pathlib
+
+    from OCC.Core.IFSelect import IFSelect_RetError
+    from OCC.Core.Interface import Interface_Static_SetCVal
+    from OCC.Core.STEPConstruct import stepconstruct_FindEntity
+    from OCC.Core.STEPControl import STEPControl_AsIs, STEPControl_Writer
+    from OCC.Core.TCollection import TCollection_HAsciiString
+
+    settings = ifcopenshell.geom.settings()
+    settings.set(settings.USE_PYTHON_OPENCASCADE, True)
+    # settings.set(settings.SEW_SHELLS, False)
+    # settings.set(settings.WELD_VERTICES, True)
+    # settings.set(settings.INCLUDE_CURVES, False)
+    settings.set(settings.USE_WORLD_COORDS, True)
+    # settings.set(settings.VALIDATE_QUANTITIES, False)
+
+    f = ifcopenshell.open(ifc_file)
+    writer = STEPControl_Writer()
+    fp = writer.WS().TransferWriter().FinderProcess()
+
+    Interface_Static_SetCVal("write.step.schema", step_schema)
+    Interface_Static_SetCVal("write.precision.mode", "1")
+    Interface_Static_SetCVal("write.step.assembly", str(step_assembly_mode))
+    iterator = ifcopenshell.geom.iterator(settings, f)
+    iterator.initialize()
+
+    while True:
+        shape = iterator.get()
+        if shape:
+            geom = shape.geometry
+            name = f.by_id(shape[0].id).Name
+            Interface_Static_SetCVal("write.step.product.name", name)
+            writer.Transfer(geom, STEPControl_AsIs)
+            item = stepconstruct_FindEntity(fp, geom)
+            if not item:
+                logger.debug("STEP item not found for FindEntity")
+            else:
+                item.SetName(TCollection_HAsciiString(name))
+        if not iterator.next():
+            break
+
+    destination_file = pathlib.Path(stp_file).with_suffix(".stp")
+    os.makedirs(destination_file.parent, exist_ok=True)
+
+    status = writer.Write(str(destination_file))
+    if int(status) > int(IFSelect_RetError):
+        raise Exception("Error during write operation")
+
+    print(f'step file created at "{destination_file}"')
