@@ -6,8 +6,11 @@ from typing import TYPE_CHECKING
 
 import numpy as np
 
+from ada.base.units import Units
+
 if TYPE_CHECKING:
     from ada import Material
+
 from .plasticity_models import PlasticityModel
 
 
@@ -181,13 +184,15 @@ class Metal:
 
     @units.setter
     def units(self, value):
-        if self._units == "m" and value == "mm":
+        if isinstance(value, str):
+            value = Units.from_str(value)
+        if self._units == Units.M and value == Units.MM:
             self._E *= 1e-6
             if self._sig_y is not None:
                 self._sig_y *= 1e-6
             self._rho *= 1e-9
             self._units = value
-        elif self._units == "mm" and value == "m":
+        elif self._units == Units.MM and value == Units.M:
             self._E *= 1e6
             if self._sig_y is not None:
                 self._sig_y *= 1e6
