@@ -1,7 +1,8 @@
-import logging
+from __future__ import annotations
+
 import math
 import pathlib
-from typing import TYPE_CHECKING, List, Tuple, Union
+from typing import TYPE_CHECKING, List, Union
 
 import numpy as np
 from OCC.Core.Bnd import Bnd_Box
@@ -42,6 +43,7 @@ from OCC.Extend.TopologyUtils import TopologyExplorer
 from ada.concepts.primitives import Penetration
 from ada.concepts.stru_beams import Beam
 from ada.concepts.transforms import Placement, Rotation
+from ada.config import get_logger
 from ada.core.utils import roundoff, tuple_minus
 from ada.core.vector_utils import unit_vector, vector_length
 from ada.fem.shapes import ElemType
@@ -53,6 +55,9 @@ from .exceptions.geom_creation import (
 
 if TYPE_CHECKING:
     from ada import Part
+    from ada.core.vector_utils import EquationOfPlane, Plane
+
+logger = get_logger()
 
 
 def extract_shapes(step_path, scale, transform, rotate, include_shells=False):
@@ -73,9 +78,8 @@ def extract_shapes(step_path, scale, transform, rotate, include_shells=False):
 
 
 def transform_shape(
-    shape: Union[TopoDS_Shape], scale=None, transform: Union[Placement, tuple, list] = None, rotate: Rotation = None
+    shape: TopoDS_Shape, scale=None, transform: Placement | tuple | list = None, rotate: Rotation = None
 ) -> TopoDS_Shape:
-
     trsf = gp_Trsf()
     if scale is not None:
         trsf.SetScaleFactor(scale)
@@ -140,7 +144,7 @@ def make_wire_from_points(points):
     return make_wire([BRepBuilderAPI_MakeEdge(gp_Pnt(*p1), gp_Pnt(*p2)).Edge()])
 
 
-def get_boundingbox(shape: TopoDS_Shape, tol=1e-6, use_mesh=True) -> Tuple[tuple, tuple]:
+def get_boundingbox(shape: TopoDS_Shape, tol=1e-6, use_mesh=True) -> tuple[tuple, tuple]:
     """
 
     :param shape: TopoDS_Shape or a subclass such as TopoDS_Face the shape to compute the bounding box from
@@ -478,7 +482,7 @@ def make_edge(p1, p2):
     res = BRepBuilderAPI_MakeEdge(p1, p2).Edge()
 
     if res.IsNull():
-        logging.debug("Edge creation returned None")
+        logger.debug("Edge creation returned None")
 
     return res
 
@@ -538,6 +542,21 @@ def make_ori_vector(
     return Part(name, units=units) / (o_shape, x_vec_shape, y_vec_shape, z_vec_shape)
 
 
+def make_eq_plane_object(name, eq_plane: EquationOfPlane, p_dist=1, plane: Plane = None, colour="white") -> Part:
+    from ada import Plate
+    from ada.core.vector_utils import Plane
+
+    if plane is None:
+        plane = Plane.XY
+
+    csys = eq_plane.get_lcsys()
+    points = eq_plane.get_points_in_lcsys_plane(p_dist=p_dist, plane=plane)
+    ori_vec_model = make_ori_vector(name=name, origin=eq_plane.point_in_plane, csys=csys)
+
+    ori_vec_model.add_plate(Plate("Surface", points, 0.001, use3dnodes=True, colour=colour, opacity=0.3))
+    return ori_vec_model
+
+
 def visualize_elem_ori(elem):
     """
 
@@ -626,7 +645,7 @@ def compute_minimal_distance_between_shapes(shp1, shp2) -> BRepExtrema_DistShape
 
     assert dss.IsDone()
 
-    logging.info("Minimal distance between shapes: ", dss.Value())
+    logger.info("Minimal distance between shapes: ", dss.Value())
 
     return dss
 
@@ -668,12 +687,12 @@ def sweep_pipe(edge, xvec, r, wt, geom_repr=ElemType.SOLID):
             o = make_circular_sec_wire(point, direction, r)
             elbow_o = BRepOffsetAPI_MakePipe(wire, o).Shape()
     except RuntimeError as e:
-        logging.error(f'Pipe sweep failed: "{e}"')
+        logger.error(f'Pipe sweep failed: "{e}"')
         return wire
     if geom_repr == ElemType.SOLID:
         boolean_result = BRepAlgoAPI_Cut(elbow_o, elbow_i).Shape()
         if boolean_result.IsNull():
-            logging.debug("Boolean returns None")
+            logger.debug("Boolean returns None")
     else:
         boolean_result = elbow_o
 
@@ -795,7 +814,7 @@ def create_beam_geom(beam: Beam, solid=True):
         try:
             assert isinstance(sec, list)
         except AssertionError as e:
-            logging.error(e)
+            logger.error(e)
         sec_result = sec
         tap_result = tap
 
