@@ -1,4 +1,5 @@
-import logging
+from __future__ import annotations
+
 import uuid
 from dataclasses import dataclass
 from itertools import chain
@@ -29,13 +30,15 @@ from pythreejs import (
     PointsMaterial,
 )
 
+from ada.config import get_logger
 from ada.fem import Elem
 
 if TYPE_CHECKING:
-    from ada import Beam, Part, Pipe, Plate, Shape, Wall
+    from ada import Beam, Part, Pipe, PipeSegElbow, PipeSegStraight, Plate, Shape, Wall
     from ada.concepts.connections import JointBase
 
 __all__ = ["MyRenderer", "SectionRenderer"]
+logger = get_logger()
 
 
 class MyRenderer(JupyterRenderer):
@@ -188,7 +191,7 @@ class MyRenderer(JupyterRenderer):
         ]
         self._fem_refs[part.fem.name] = (part.fem, edge_geometry)
 
-    def DisplayAdaShape(self, shp: "Shape"):
+    def DisplayAdaShape(self, shp: Shape):
         res = self.DisplayShape(
             shp.geom,
             transparency=shp.transparent,
@@ -199,7 +202,7 @@ class MyRenderer(JupyterRenderer):
         for r in res:
             self._refs[r.name] = shp
 
-    def DisplayBeam(self, beam: "Beam"):
+    def DisplayBeam(self, beam: Beam):
         try:
             if "ifc_file" in beam.metadata.keys():
                 from ada.ifc.read.read_shapes import get_ifc_geometry
@@ -212,7 +215,7 @@ class MyRenderer(JupyterRenderer):
                 geom = beam.solid
             res = self.DisplayShape(geom, shape_color=beam.colour, render_edges=True)
         except BaseException as e:
-            logging.debug(f'Unable to create solid for "{beam.name}" due to {e}')
+            logger.debug(f'Unable to create solid for "{beam.name}" due to {e}')
             return None
 
         for r in res:
@@ -224,21 +227,27 @@ class MyRenderer(JupyterRenderer):
         try:
             res = self.DisplayShape(geom, shape_color=plate.colour_webgl, opacity=0.5)
         except BaseException as e:
-            logging.error(e)
+            logger.error(e)
             return None
 
         for r in res:
             self._refs[r.name] = plate
 
-    def DisplayPipe(self, pipe: "Pipe"):
-        # self.AddShapeToScene(geom)
+    def DisplayPipeSeg(self, pipe_seg: PipeSegElbow | PipeSegStraight):
+        res = []
+        res += self.DisplayShape(pipe_seg.solid, shape_color=pipe_seg.parent.colour_webgl, opacity=0.5)
+
+        for r in res:
+            self._refs[r.name] = pipe_seg
+
+    def DisplayPipe(self, pipe: Pipe):
         res = []
 
         for i, geom in enumerate([x.solid for x in pipe.segments]):
             try:
                 res += self.DisplayShape(geom, shape_color=pipe.colour_webgl, opacity=0.5)
             except BaseException as e:
-                logging.error(e)
+                logger.error(e)
                 continue
 
         for r in res:
@@ -248,7 +257,7 @@ class MyRenderer(JupyterRenderer):
         try:
             res = self.DisplayShape(wall.solid, shape_color=wall.colour, opacity=0.5)
         except BaseException as e:
-            logging.error(e)
+            logger.error(e)
             return None
 
         for r in res:
@@ -285,7 +294,7 @@ class MyRenderer(JupyterRenderer):
         list(filter(None, map(self.DisplayBeam, all_beams)))
 
     def DisplayObj(self, obj):
-        from ada import Beam, Part, Pipe, Plate, Shape
+        from ada import Beam, Part, Pipe, PipeSegElbow, PipeSegStraight, Plate, Shape
         from ada.concepts.connections import JointBase
 
         if issubclass(type(obj), Part) is True:
@@ -296,6 +305,8 @@ class MyRenderer(JupyterRenderer):
             self.DisplayPlate(obj)
         elif type(obj) is Pipe:
             self.DisplayPipe(obj)
+        elif isinstance(obj, (PipeSegStraight, PipeSegElbow)):
+            self.DisplayPipeSeg(obj)
         elif issubclass(type(obj), JointBase):
             self.display_joint(obj)
         elif issubclass(type(obj), Shape):
@@ -710,6 +721,10 @@ class MyRenderer(JupyterRenderer):
             html_value += table_str
         elif type(selected_part) is Pipe:
             html_value += ", <b>Type:</b> Pipe<br>"
+        elif isinstance(selected_part, PipeSegStraight):
+            html_value += ", <b>Type:</b> PipeSegStraight<br>"
+        elif isinstance(selected_part, PipeSegElbow):
+            html_value += ", <b>Type:</b> PipeSegElbow<br>"
         elif type(selected_part) is Wall:
             html_value += ", <b>Type:</b> Wall<br>"
         else:
@@ -742,7 +757,7 @@ class MyRenderer(JupyterRenderer):
         elif setref in fem.nsets.keys():
             print(f'Set "{setref}" is a node set (which is not yet supported)')
         else:
-            logging.error(f'Unrecognized set "{setref}". Not belonging to node or elements')
+            logger.error(f'Unrecognized set "{setref}". Not belonging to node or elements')
         edge_geom.attributes["color"].array = color_array
 
     def highlight_elem(self, elem_id, fem_name):
